#!/bin/sh

echo "===> Running "`basename $0`

sysctl compat.linux.osname > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# Linux module loaded.
	echo " - Linux module loaded"
else
	# Linux module not loaded.
	echo " * Linux module not loaded, loading."
	kldload linux > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -eq 0 ]; then
		echo " - Module loaded."
	else
		echo "ERROR: Error loading module!"
		echo "ERROR: Compile and install the Linux module."
		exit 1
	fi

	grep linux_enable /etc/rc.conf > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -eq 0 ]; then
		echo " - linux_enable present."
		linux_opt=`grep linux_enable /etc/rc.conf | awk -F \" '{print $2}'`

		case ${linux_opt} in
		[Yy][Ee][Ss])
			echo " - linux_enable enabled."
			;;
		[Nn][Oo])
			echo " * linux_enable disabled, enabling"
			sed -e s/linux_enable.*/linux_enable=\"YES\"/g -i.orig /etc/rc.conf
			;;
		*)
			echo " * linux_enable unknown state, enabling"
			sed -e s/linux_enable.*/linux_enable=\"YES\"/g -i.orig /etc/rc.conf
			;;
		esac

	else
		echo " * linux_enable not present, adding"
		echo 'linux_enable="YES"' >> /etc/rc.conf
	fi
fi

kldstat -v -n kernel | grep linux > /dev/null 2>&1 ; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# linux compat module built into kernel, do nothing.
else
	grep linux_load /boot/loader.conf > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -eq 0 ]; then
		# Present.
		echo " - linux_load present"
		nvidia_opt=`grep linux_load /boot/loader.conf | awk -F \" '{print $2}'`

		case ${nvidia_opt} in
		[Yy][Ee][Ss])
			echo " - linux_load enabled."
			;;
		[Nn][Oo])
			echo " * linux_load disabled, enabling"
			sed -e s/linux_load.*/linux_load=\"YES\"/g -i.orig /boot/loader.conf
			;;
		*)
			echo " * linux_load unknown state, enabling"
			sed -e s/linux_load.*/linux_load=\"YES\"/g -i.orig /boot/loader.conf
			;;
		esac
	
	else
		# Not present
		echo " * linux_load not present, adding."
		echo 'linux_load="YES"' >> /boot/loader.conf
	fi
fi

grep nvidia_load /boot/loader.conf > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	# Present.
	echo " - nvidia_load present"
	nvidia_opt=`grep nvidia_load /boot/loader.conf | awk -F \" '{print $2}'`

	case ${nvidia_opt} in
	[Yy][Ee][Ss])
		echo " - nvidia_load enabled."
		;;
	[Nn][Oo])
		echo " * nvidia_load disabled, enabling"
		sed -e s/nvidia_load.*/nvidia_load=\"YES\"/g -i.orig /boot/loader.conf
		;;
	*)
		echo " * nvidia_load unknown state, enabling"
		sed -e s/nvidia_load.*/nvidia_load=\"YES\"/g -i.orig /boot/loader.conf
		;;
	esac

else
	# Not present
	echo " * nvidia_load not present, adding."
	echo 'nvidia_load="YES"' >> /boot/loader.conf
fi

kldstat -n nvidia > /dev/null 2>&1; RESULT=$?
if [ ${RESULT} -eq 0 ]; then
	echo ' * NVIDIA module loaded, unloading.'
	kldunload -n nvidia > /dev/null 2>&1; RESULT=$?
	if [ ${RESULT} -ne 0 ]; then
		echo 'ERROR: Failed to unload NVIDIA module!'
		echo 'ERROR: Is the module in use?'
		exit 1;
	fi
fi

echo ' * Loading the NVIDIA module.'
kldload nvidia > /dev/null 2>&1 ; RESULT=$?
if [ ${RESULT} -ne 0 ]; then
	echo 'ERROR: Failed to load the nvidia module!'
	exit 1;
fi
echo ' - NVIDIA module loaded.'
